<?php

/**
 * 
 * @file This file concentrates all general functionality related to Cases in 
 * Case Tracker, leaving to the other files in /admin the stuff related to CRUD
 * and structural configuration
 */

/**
 * Determines whether the given user has access to a scrum_sprint.
 *
 * @param $op
 *   The operation being performed. One of 'view', 'update', 'create', 'delete'
 *   or just 'edit' (being the same as 'create' or 'update').
 * @param $sprint
 *   Optionally a scrum_sprint or a scrum_sprint type to check 
 *   access for. If nothing is given, access for all scrum_sprints 
 *   is determined.
 * @param $account
 *   The user to check for. Leave it to NULL to check for the global user.
 * @return boolean
 *   Whether access is allowed or not.
 */
function scrum_sprint_access($op, $sprint = NULL, $account = NULL) {
  if (user_access('administer scrum_sprints', $account)) {
    return TRUE;
  }
  if (isset($sprint) && is_object($sprint) && $type_name = $sprint->type) {
    $op = ($op == 'view') ? 'view' : 'edit';
    if (user_access("$op any $type_name scrum_sprint", $account)) {
      return TRUE;
    }
  }
  return FALSE;
}

/**
 * Fetch a scrum_sprint object. Make sure that the wildcard you choose 
 * in the scrum_sprint entity definition fits the function name here.
 *
 * @param $sid
 *   Integer specifying the scrum_sprint id.
 * @param $reset
 *   A boolean indicating that the internal cache should be reset.
 * @return
 *   A fully-loaded $sprint object or FALSE if it cannot be loaded.
 *
 * @see scrum_sprint_load_multiple()
 */
function scrum_sprint_load($sid, $reset = FALSE) {
  $cases = scrum_sprint_load_multiple(array($sid), array(), $reset);
  return reset($cases);
}

/**
 * Load multiple scrum_sprints based on certain conditions.
 *
 * @param $sids
 *   An array of sprint IDs.
 * @param $conditions
 *   An array of conditions to match against the {scrum_sprint} table.
 * @param $reset
 *   A boolean indicating that the internal cache should be reset.
 * @return
 *   An array of scrum_sprint objects, indexed by sid.
 *
 * @see entity_load()
 * @see scrum_sprint_load()
 */
function scrum_sprint_load_multiple($sids = array(), $conditions = array(), $reset = FALSE) {
  return entity_load('scrum_sprint', $sids, $conditions, $reset);
}

/**
 * Deletes a scrum_sprint.
 */
function scrum_sprint_delete(ScrumSprint $sprint) {
  $sprint->delete();
}

/**
 * Delete multiple scrum_sprints.
 *
 * @param $sids
 *   An array of scrum_sprint IDs.
 */
function scrum_sprint_delete_multiple(array $sids) {
  entity_get_controller('scrum_sprint')->delete($sids);
}

/**
 * Create a scrum_sprint object.
 */
function scrum_sprint_create($values = array()) {
  return entity_get_controller('scrum_sprint')->create($values);
}

/**
 * Saves a scrum_sprint to the database.
 *
 * @param $scrum_sprint
 *   The scrum_sprint object.
 */
function scrum_sprint_save(ScrumSprint $sprint) {
  return $sprint->save();
}

/**
 * URI callback for scrum_sprints
 */
function scrum_sprint_uri(ScrumSprint $sprint) {
  return $sprint->defaultUri();
}

/**
 * Menu title callback for showing individual entities
 */
function scrum_sprint_page_title(ScrumSprint $sprint) {
  return $sprint->title;
}

/**
 * Sets up content to show an individual scrum_sprint
 * @todo - get rid of drupal_set_title();
 */
function scrum_sprint_page_view(ScrumSprint $sprint, $view_mode = 'full') {

  drupal_set_title($sprint->title . ' (#ID ' . $sprint->sid . ')');
  $project_wrapper = casetracker_project_load($sprint->project_id);
  casetracker_project_set_breadcrumb(array($project_wrapper->title));

  return entity_view('scrum_sprint', array($sprint->sid => $sprint), $view_mode, NULL, TRUE);
}

/**
 * The class used for sprint entities
 */
class ScrumSprint extends Entity {

  public function __construct($values = array()) {
    parent::__construct($values, 'scrum_sprint');
  }

  protected function defaultLabel() {
    return $this->title;
  }

  protected function defaultUri() {
    return array('path' => 'scrum/sprint/' . $this->sid);
  }

}

/**
 * The Controller for CaseTrackerCase entities
 */
class ScrumSprintController extends EntityAPIController {

  public function __construct($entityType) {
    parent::__construct($entityType);
  }

  /**
   * 
   * @param $type
   *   The machine-readable type of the sprint.
   *
   * @return
   *   A sprint object with all default fields initialized.
   */
  public function create(array $values = array()) {
    // Add values that are specific to our Sprint
    $values += array(
      'pid' => '',
      'is_new' => TRUE,
      'title' => '',
      'uid' => '',
      'project_id' => '',
      'created' => '',
      'changed' => '',
    );

    $sprint = parent::create($values);
    return $sprint;
  }

}